// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/HPDI32/HPDI32_Linux_sdk_6.x.x_GSC_DN/api/dev_cable.c $
// $Rev: 30377 $
// $Date: 2014-09-18 12:09:04 -0500 (Thu, 18 Sep 2014) $

#include "dev_cable.h"

#include "dev_config.h"
#include "dev_reg.h"



/******************************************************************************
*
*	Function:	cable_clock_state
*
*	Purpose:
*
*		Implement the Cable Clock State parameter service. Validate and apply
*		the setting, as appropriate, then retrieve the current setting for
*		the caller, also as appropriate.
*
*	Arguments:
*
*		dev			The device to access.
*
*		which		A field of bits specifying the subjects to act on.
*
*		set			The value to apply.
*
*		get			Retriev the current setting an record it here, if not
*					NULL.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		A status reflecting the error encountered.
*
******************************************************************************/

U32 cable_clock_state(dev_data_t* dev, U32 which, UL set, UL* get)
{
	U32	bcr;
	U32	status;

	for (;;)	// A convenience loop.
	{
		status	= config_which_validate(dev, which, 0);
		if (status != (U32) GSC_SUCCESS) break;

		// SET is not supported for this parameter.

		status	= GSC_REG_READ(dev, HPDI32_BCR, &bcr);
		if (status != (U32) GSC_SUCCESS) break;

		if (get)
		{
			if (bcr & (HPDI32_BCR_TX_ENABLED | HPDI32_BCR_RX_ENABLED))
				get[0]	= HPDI32_CABLE_CLOCK_STATE_ACTIVE;
			else
				get[0]	= HPDI32_CABLE_CLOCK_STATE_INACTIVE;
		}

		break;
	}

	return(status);
}



/******************************************************************************
*
*	Function:	_command_mode_line
*
*	Purpose:
*
*		Implement the Cable Command Mode parameter service for a single cable
*		command signal. Validate and apply the setting, as appropriate, then
*		retrieve the current setting for the caller, also as appropriate.
*
*	Arguments:
*
*		dev			The device to access.
*
*		index		The index of the command signal to access.
*
*		set			The value to apply.
*
*		get			Retrieve the current setting and record it here, if not
*					NULL.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		A status reflecting the error encountered.
*
******************************************************************************/

static U32 _command_mode_line(dev_data_t* dev, int index, UL set, UL* get)
{
	U32	bcr;
	int	config;
	U32	mask;
	U32	status	= (U32) GSC_FAILED;

	for (;;)	// A convenience loop.
	{
		if (index == 0)
			config	= (dev->cache.fsr & HPDI32_FSR_GPIO6_TXAS) ? 1 : 0;
		else
			config	= (dev->cache.fsr & HPDI32_FSR_GPIO_0_5) ? 1 : 0;

		// SET

		if (set == GSC_NO_CHANGE)
		{
		}
		else if (set == GSC_DEFAULT)
		{
			set	= HPDI32_CABLE_COMMAND_MODE_DEFAULT;
		}
		else if (set == HPDI32_CABLE_COMMAND_MODE_FLOW_CONTROL)
		{
		}
		else if (	(set == HPDI32_CABLE_COMMAND_MODE_GPIO_IN) ||
					(set == HPDI32_CABLE_COMMAND_MODE_GPIO_OUT_HI) ||
					(set == HPDI32_CABLE_COMMAND_MODE_GPIO_OUT_LOW))
		{
		}
		else if (dev->strict_args)
		{
			status	= (U32) GSC_INVALID_DATA;
			break;
		}
		else
		{
			set	= GSC_NO_CHANGE;
		}

		if (set != GSC_NO_CHANGE)
		{
			if (set == HPDI32_CABLE_COMMAND_MODE_FLOW_CONTROL)
				set	= HPDI32_BCR_C0_FV << index;
			else if (set == HPDI32_CABLE_COMMAND_MODE_GPIO_IN)
				set	= HPDI32_BCR_C0_IN << index;
			else if (set == HPDI32_CABLE_COMMAND_MODE_GPIO_OUT_HI)
				set	= HPDI32_BCR_C0_OUT_H << index;
			else if (set == HPDI32_CABLE_COMMAND_MODE_GPIO_OUT_LOW)
				set	= HPDI32_BCR_C0_OUT_L << index;

			mask	= HPDI32_BCR_C0_MASK << index;
			status	= GSC_REG_MOD(dev, HPDI32_BCR, set, mask);
			if (status != (U32) GSC_SUCCESS) break;
		}

		if (get)
		{
			if (config)
			{
				status	= GSC_REG_READ(dev, HPDI32_BCR, &bcr);
				if (status != (U32) GSC_SUCCESS) break;

				bcr		>>= index;
				mask	= bcr & HPDI32_BCR_C0_MASK;

				if (mask == HPDI32_BCR_C0_FV)
					get[0]	= HPDI32_CABLE_COMMAND_MODE_FLOW_CONTROL;
				else if (mask == HPDI32_BCR_C0_IN)
					get[0]	= HPDI32_CABLE_COMMAND_MODE_GPIO_IN;
				else if (mask == HPDI32_BCR_C0_OUT_H)
					get[0]	= HPDI32_CABLE_COMMAND_MODE_GPIO_OUT_HI;
				else if (mask == HPDI32_BCR_C0_OUT_L)
					get[0]	= HPDI32_CABLE_COMMAND_MODE_GPIO_OUT_LOW;
			}
			else
			{
				get[0]	= HPDI32_CABLE_COMMAND_MODE_FLOW_CONTROL;
			}
		}

		break;
	}

	return(status);
}



/******************************************************************************
*
*	Function:	cable_command_mode
*
*	Purpose:
*
*		Implement the Cable Command Mode parameter service. Validate and apply
*		the setting, as appropriate, then retrieve the current setting for
*		the caller, also as appropriate.
*
*	Arguments:
*
*		dev			The device to access.
*
*		which		A field of bits specifying the subjects to act on.
*
*		set			The value to apply.
*
*		get			Retrieve the current setting and record it here, if not
*					NULL.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		A status reflecting the error encountered.
*
******************************************************************************/

U32 cable_command_mode(dev_data_t*dev, U32 which, UL set, UL* get)
{
	int	i;
	U32	mask;
	U32	stat;
	U32	status;

	status	= config_which_validate(dev, which, HPDI32_WHICH_COMMAND_ALL_);

	if (status == (U32) GSC_SUCCESS)
	{
		for (i = 0; i <= 6; i++)
		{
			mask	= 0x1 << i;

			if (which & mask)
			{
				stat	= _command_mode_line(dev, i, set, get);
				status	= (status == (U32) GSC_SUCCESS) ? stat : status;
			}
		}
	}

	return(status);
}



/******************************************************************************
*
*	Function:	_command_state_line
*
*	Purpose:
*
*		Implement the Cable Command State parameter service for a single cable
*		command signal. Retrieve the current setting for the caller, also as
*		appropriate.
*
*	Arguments:
*
*		dev			The device to access.
*
*		index		The index of the command signal to access.
*
*		set			The value to apply.
*
*		get			Retrieve the current setting and record it here, if not
*					NULL.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		A status reflecting the error encountered.
*
******************************************************************************/

static U32 _command_state_line(dev_data_t* dev, int index, UL set, UL* get)
{
	U32	bcr;
	U32	bsr;
	int	config;
	int	fc;
	U32	mask;
	U32	status	= (U32) GSC_FAILED;

	for (;;)	// A convenience loop.
	{
		if (index == 0)
			config	= (dev->cache.fsr & HPDI32_FSR_GPIO6_TXAS) ? 1 : 0;
		else
			config	= (dev->cache.fsr & HPDI32_FSR_GPIO_0_5) ? 1 : 0;

		// SET not supported for this parameter.

		if (get)
		{
			status	= GSC_REG_READ(dev, HPDI32_BCR, &bcr);
			if (status != (U32) GSC_SUCCESS) break;
			fc	= ((bcr >> index) & HPDI32_BCR_C0_MASK) == HPDI32_BCR_C0_FV;

			if ((config == 0) || (fc))
			{
				// Flow Control
				//
				// The state is not driven by the line level which is not
				// always readable. The state reported is whether the line
				// is driven, or should be driven.

				switch (index)
				{
					default:
					case 0:		// Frame Valid
					case 1:		// Line Valid
					case 2:		// Status Valid
					case 3:		// Rx Ready
					case 4:		// Tx Ready
								mask	= HPDI32_BCR_TX_ENABLED
										| HPDI32_BCR_RX_ENABLED;
								break;

					case 5:		// Tx Enabled
								mask	= HPDI32_BCR_TX_ENABLED;
								break;

					case 6:		// Rx Enabled
								mask	= HPDI32_BCR_RX_ENABLED;
								break;
				}

				if (mask & bcr)
					get[0]	= HPDI32_CABLE_COMMAND_STATE_ACTIVE;
				else
					get[0]	= HPDI32_CABLE_COMMAND_STATE_INACTIVE;
			}
			else
			{
				status	= GSC_REG_READ(dev, HPDI32_BSR, &bsr);
				if (status != (U32) GSC_SUCCESS) break;
				bsr	= bsr >> index;

				if (bsr & 0x1)
					get[0]	= HPDI32_CABLE_COMMAND_STATE_ACTIVE;	// GPIO HIGH.
				else
					get[0]	= HPDI32_CABLE_COMMAND_STATE_INACTIVE;	// GPIO LOW.
			}
		}

		break;
	}

	return(status);
}



/******************************************************************************
*
*	Function:	cable_command_state
*
*	Purpose:
*
*		Implement the Cable Command State parameter service. Validate and
*		apply the setting, as appropriate, then retrieve the current setting
*		for the caller, also as appropriate.
*
*	Arguments:
*
*		dev			The device to access.
*
*		which		A field of bits specifying the subjects to act on.
*
*		set			The value to apply.
*
*		get			Retrieve the current setting and record it here, if not
*					NULL.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		A status reflecting the error encountered.
*
******************************************************************************/

U32 cable_command_state(dev_data_t*dev, U32 which, UL set, UL* get)
{
	int	i;
	U32	mask;
	U32	stat;
	U32	status;

	status	= config_which_validate(dev, which, HPDI32_WHICH_COMMAND_ALL_);

	if (status == (U32) GSC_SUCCESS)
	{
		for (i = 0; i <= 6; i++)
		{
			mask	= 0x1 << i;

			if (which & mask)
			{
				stat	= _command_state_line(dev, i, set, get);
				status	= (status == (U32) GSC_SUCCESS) ? stat : status;
			}
		}
	}

	return(status);
}


