// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/HPDI32/HPDI32_Linux_sdk_6.x.x_GSC_DN/api/gsc_os_device.c $
// $Rev: 30377 $
// $Date: 2014-09-18 12:09:04 -0500 (Thu, 18 Sep 2014) $

#include <ctype.h>
#include <errno.h>
#include <fcntl.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>

#include "gsc_os_device.h"
#include "gsc_os_event.h"
#include "gsc_os_error.h"




/******************************************************************************
*
*	Function:	_device_t_init
*
*	Purpose:
*
*		Initialize the given device structure.
*
*	Arguments:
*
*		device		The structure to initialize.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
******************************************************************************/

static U32 _device_t_init(os_device_t* device)
{
	U32	status;

	if (device == NULL)
	{
		status	= GSC_NULL_PARAM;
	}
	else
	{
		status	= GSC_SUCCESS;
		memset(device, 0, sizeof(os_device_t));
		OS_KEY_SET(device, OS_DEVICE_T_KEY);
		OS_DEVICE_T_KEY_VERIFY(device);
	}

	return(status);
}


/******************************************************************************
*
*	Function:	os_device_close
*
*	Purpose:
*
*		Close a channel to the specified device.
*
*	Arguments:
*
*		device		The device to close.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
******************************************************************************/

U32 os_device_close(os_device_t* device)
{
	U32	status;

	if (device)
	{
		status	= GSC_SUCCESS;
		OS_DEVICE_T_KEY_VERIFY(device);

#if DEV_USES_IRQ

		OS_SEM_T_KEY_VERIFY(&device->events.sem);
		os_events_release(device);
		os_sem_destroy(&device->events.sem);

#elif ! defined(DEV_USES_IRQ)
	#error DEV_USES_IRQ undefined. Must #include dev_drv.h.
#endif

		close(device->fd);
		memset(device, 0, sizeof(os_device_t));
	}
	else
	{
		status	= GSC_NULL_PARAM;
	}

	return(status);
}


/******************************************************************************
*
*	Function:	os_device_hc_base_index
*
*	Purpose:
*
*		Report to the caller if the base index is hard coded.
*		Under Linux, it is.
*
*	Arguments:
*
*		hc			Report here if it is hard coded. 0 = NO, !0 = YES
*
*		base_index	Report the base index here as zero.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
******************************************************************************/

U32 os_device_hc_base_index(int* hc, U8* base_index)
{
	U32	status;

	if ((hc) && (base_index))
	{
		status			= GSC_SUCCESS;
		hc[0]			= 1;
		base_index[0]	= 0;
	}
	else
	{
		status	= GSC_NULL_PARAM;
	}

	return(status);
}



/******************************************************************************
*
*	Function:	os_device_open
*
*	Purpose:
*
*		Open a channel to the specified device.
*
*	Arguments:
*
*		index		The index of the board to access.
*
*		name		The product name of interest.
*
*		device		The resulting access is recorded here.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
******************************************************************************/

U32 os_device_open(int index, const char* name, os_device_t* device)
{
	int		fd;
	char	path[64];
	char*	psz;
	U32		status	= GSC_SUCCESS;

	for (;;)	// A convenience loop.
	{
		if ((name == NULL) || (device == NULL))
		{
			status	= GSC_NULL_PARAM;
			break;
		}

		if (index < 0)
		{
			status	= GSC_INVALID_INDEX;
			break;
		}

		_device_t_init(device);
		OS_DEVICE_T_KEY_VERIFY(device);
		sprintf(path, "/dev/%s-%d", name, (int) index);

		for (psz = path; psz[0]; psz++)
		{
			if (isupper(psz[0]))
				psz[0]	+= 'a' - 'A';
		}

		fd	= open(path, O_RDWR);

		if (fd == -1)
		{
			device->fd	= 0;
			status		= os_errno_2_status(errno);
		}
		else
		{
			device->fd	= fd;

#if DEV_USES_IRQ

			status		= os_sem_create(&device->events.sem, "event list");

			if (status != GSC_SUCCESS)
				os_device_close(device);

#elif ! defined(DEV_USES_IRQ)
	#error DEV_USES_IRQ undefined. Must #include dev_drv.h.
#endif
		}

		OS_DEVICE_T_KEY_VERIFY(device);
		break;
	}

	return(status);
}


